(async () => {
  const data = await d3.csv(
    'https://raw.githubusercontent.com/HemanthKakani1407/CSCE-5320-Task-8/main/ElectricCarData_Clean.csv',
  );

  // Group the data by brand and find the highest price for each brand
  const brands = Array.from(
    d3.group(data, (d) => d.Brand),
    ([key, value]) => {
      return {
        Brand: key,
        PriceEuro: d3.max(value, (d) => +d.PriceEuro),
      };
    },
  );

  // Sort the brands by price in ascending order
  brands.sort((a, b) => a.PriceEuro - b.PriceEuro);

  const margin = {
    top: 50,
    right: 50,
    bottom: 50,
    left: 100,
  };
  const width = 800 - margin.left - margin.right;
  const height = 500 - margin.top - margin.bottom;

  const svg = d3
    .select('#chart')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left}, ${margin.top})`,
    );

  const maxPrice = d3.max(brands, (d) => +d.PriceEuro);

  const xScale = d3
    .scaleBand()
    .domain(brands.map((d) => d.Brand))
    .range([0, width])
    .padding(0.1);

  const yScale = d3
    .scaleLinear()
    .domain([0, maxPrice])
    .range([height, 0]);

  const radiusScale = d3
    .scaleSqrt()
    .domain([0, maxPrice])
    .range([2, 20]);

  const colorScale = d3
    .scaleSequential(d3.interpolatePlasma)
    .domain([0, maxPrice]);

  svg
    .selectAll('circle')
    .data(brands)
    .enter()
    .append('circle')
    .attr(
      'cx',
      (d) => xScale(d.Brand) + xScale.bandwidth() / 2,
    )
    .attr('cy', (d) => yScale(+d.PriceEuro))
    .attr('r', (d) => radiusScale(+d.PriceEuro))
    .style('fill', (d) => colorScale(+d.PriceEuro))
    .style('opacity', 0.7);

  const xAxis = d3
    .axisBottom(xScale)
    .tickSizeOuter(0)
    .tickPadding(10);
  svg
    .append('g')
    .attr('transform', `translate(0, ${height})`)
    .call(xAxis)
    .selectAll('text')
    .style('text-anchor', 'end')
    .attr('dx', '-.8em')
    .attr('dy', '.15em')
    .attr('transform', 'rotate(-45)');

  const yAxis = d3
    .axisLeft(yScale)
    .tickSizeOuter(0)
    .tickPadding(10);
  svg.append('g').call(yAxis);

  svg
    .append('text')
    .attr('x', width / 2)
    .attr('y', height + margin.top)
    .attr('text-anchor', 'middle')
    .text('Brands');

  svg
    .append('text')
    .attr('transform', 'rotate(-90)')
    .attr('x', -height / 2)
    .attr('y', -margin.left * 0.8)
    .attr('text-anchor', 'middle')
    .text('Price (Euro)');

  svg
    .append('text')
    .attr('x', width / 2)
    .attr('y', 0 - margin.top / 2)
    .attr('text-anchor', 'middle')
    .style('font-size', '16px')
    .style('text-decoration', 'underline')
    .text('Electric Car Prices by Brand');
})();
